@@ -1,20 +1,14 @@
 #!/bin/perl
 #
 use Carp;
-use Getopt::Long;
-use Math::Polynomial::Solve qw(:sturm);
+use Math::Polynomial::Solve qw(:sturm ascending_order);
 use strict;
 use warnings;
-#use IO::Prompt;
-
-my $line;
-my $ascending = 0;
-
-GetOptions('ascending' => \$ascending);
 
+my $ascending = 1;
 ascending_order($ascending);
 
-while ($line = prompt("Polynomial: "))
+while (my $line = prompt("Polynomial: "))
 {
 	my @coef = split(/,? /, $line);
 
@@ -25,11 +19,19 @@ while ($line = prompt("Polynomial: "))
 	my @signs = sturm_sign_chain(\@chain, \@xvals);
 
 	print "\nPolynomial: [", join(", ", @coef), "]\n";
+	print "Sturm chain:\n";
 
-	foreach my $j (0..$#signs)
+	for my $j (0..$#chain)
+	{
+		my @c = @{$chain[$j]};
+		print sprintf("    Fn%02d: [", $j) . join(", ", @c), "]\n";
+	}
+
+	for my $j (0..$#signs)
 	{
 		my @s = @{$signs[$j]};
-		print sprintf("x = %4f: [", $xvals[$j]) . join("   ", @s), "] ",
+		print sprintf("x = %4f: [", $xvals[$j]) . join("   ", @s),
+			"], sign count:  ",
 			sturm_sign_count(@s), "\n\n";
 	}
 }
