@@ -9,8 +9,9 @@ use Carp;
 use Math::Complex;
 use Math::Utils qw(:polynomial :utility);
 use Exporter;
-our @ISA = qw(Exporter);
 
+our $VERSION = '2.81';
+our @ISA = qw(Exporter);
 
 #
 # Three # for "I am here" messages.
@@ -20,8 +21,6 @@ our @ISA = qw(Exporter);
 #
 #use Smart::Comments q(######);
 
-@ISA = qw(Exporter);
-
 #
 # Export only on request.
 #
@@ -67,12 +66,10 @@ our @EXPORT_OK = (
 
 our @EXPORT = qw( ascending_order );
 
-our $VERSION = '2.80';
-
 #
-# See the END block.
+# Add an :all tag automatically.
 #
-my $ascending_order_called = 0;
+$EXPORT_TAGS{all} = [@EXPORT_OK, @EXPORT];
 
 #
 # Options to set or unset to force poly_roots() to use different
@@ -146,6 +143,12 @@ BEGIN
 #
 my $ascending_flag = 0;		# default 0, in a later version it will be 1.
 
+#
+# See the END block.
+#
+my $ascending_order_called = 0;
+my %called_by;
+
 =pod
 
 =encoding UTF-8
@@ -276,10 +279,10 @@ sub ascending_order
 # the original equation are found by taking the cube roots of each
 # root of the quadratic.
 #
-# Not exported.
+# Not exported. Coefficients are always in ascending order.
 sub poly_analysis
 {
-	my(@coefficients) = ($ascending_flag)? reverse @_: @_;
+	my(@coefficients) = @_;
 	my @czp;
 	my $m = 1;
 
@@ -288,10 +291,10 @@ sub poly_analysis
 	#
 	# Realistically I don't expect any gaps that can't be handled by
 	# the first three prime numbers, but it's not much of a waste of
-	# space to go up to 31.
+	# space to check the first dozen.
 	#
 	@czp = grep(($#coefficients % $_) == 0,
-		(2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31)
+		(2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37)
 	);
 
 	#
@@ -301,7 +304,7 @@ sub poly_analysis
 	#
 	if (@czp)
 	{
-		for my $j (0..$#coefficients)
+		for my $j (1..$#coefficients - 1)
 		{
 			if (abs($coefficients[$j]) > $epsilon)
 			{
@@ -338,14 +341,18 @@ sub poly_analysis
 
 =head1 EXPORT
 
+There is an B<all> tag that exports everything.
+
 Currently there is one default export, L<ascending_order|ascending_order()>.
 
-The remaining functions may be individually named in an export list,
-but there are also four export tags:
+If you want to have more fine-grained control you may
+individually name the functions in an export list, or
+use one four export tags:
+
 L<classical|Classical Functions>,
 L<numeric|Numeric Functions>,
-L<sturm|Sturm Functions>, and
-L<utility|Utility Functions>.
+L<sturm|Sturm Functions>,
+L<utility|Utility Functions>,
 
 =head2 EXPORTED BY DEFAULT
 
@@ -373,7 +380,7 @@ more convenient to change the default parameter list of
 Math::Polynomial::Solve's functions, using the ascending_order() function:
 
     use Math::Polynomial;
-    use Math::Polynomial::Solve qw(:classical :numeric);
+    use Math::Polynomial::Solve qw(:all);
 
     ascending_order(1);
 
@@ -428,7 +435,7 @@ ignores the settings of C<poly_options()>.
 
 sub poly_roots
 {
-	my(@coefficients) = ($ascending_flag)? reverse @_: @_;
+	my(@coefficients) = ($ascending_flag == 0)? reverse @_: @_;
 	my(@x, @zero_x);
 	my $subst_degree = 1;
 
@@ -437,8 +444,8 @@ sub poly_roots
 	#
 	# Check for zero coefficients in the higher-powered terms.
 	#
-	shift @coefficients while (scalar @coefficients and
-				   abs($coefficients[0]) < $epsilon);
+	pop @coefficients while (scalar @coefficients and
+				   abs($coefficients[$#coefficients]) < $epsilon);
 
 	if (@coefficients == 0)
 	{
@@ -450,15 +457,16 @@ sub poly_roots
 	# How about zero coefficients in the low terms?
 	#
 	while (scalar @coefficients and
-	       abs($coefficients[$#coefficients]) < $epsilon)
+	       abs($coefficients[0]) < $epsilon)
 	{
 		push @zero_x, 0;
-		pop @coefficients
+		shift @coefficients
 	}
 
 	#
-	# If the polynomial is of the form ax**n + c, and $option{root_function}
-	# is set, use the Math::Complex::root() function to return the roots.
+	# If the polynomial is of the form c + ax**n, and if the
+	# root_function option is set, use the Math::Complex::root()
+	# function to return the roots.
 	#
 	### %option
 	#
@@ -466,7 +474,7 @@ sub poly_roots
 	    poly_nonzero_term_count(@coefficients) == 2)
 	{
 		return  @zero_x,
-			root(-$coefficients[$#coefficients]/$coefficients[0],
+			root(-$coefficients[0]/$coefficients[$#coefficients],
 			     $#coefficients);
 	}
 
@@ -475,7 +483,7 @@ sub poly_roots
 	# See if we can reduce the size of the polynomial by
 	# doing some variable substitution.
 	#
-	if ($option{varsubst})
+	if ($option{varsubst} and $#coefficients > 1)
 	{
 		my $cf;
 		($cf, $subst_degree) = poly_analysis(@coefficients);
@@ -490,12 +498,13 @@ sub poly_roots
 	#### @coefficients
 	#### $subst_degree
 	#
-
 	#
-	# The following root solvers do their own coefficient
-	# reversing, so undo the earlier reversal now.
+	# With the coefficents in ascending order,
+	# pretend it was always that way for the next
+	# function calls.
 	#
-	@coefficients = reverse @coefficients if ($ascending_flag);
+	my $temp_ascending_flag = $ascending_flag;
+	$ascending_flag = 1;
 
 	if ($option{hessenberg} or $#coefficients > 4)
 	{
@@ -523,6 +532,8 @@ sub poly_roots
 		@x = linear_roots(@coefficients);
 	}
 
+	$ascending_flag = $temp_ascending_flag;
+
 	@x = map(root($_, $subst_degree), @x) if ($subst_degree > 1);
 
 	return  @zero_x, @x;
@@ -589,7 +600,7 @@ hessenberg option is zero.
 
 Currently the variable substitution is fairly simple and will only look
 for gaps of zeros in the coefficients that are multiples of the prime numbers
-less than or equal to 31 (2, 3, 5, et cetera).
+less than or equal to 37 (2, 3, 5, et cetera).
 
 =back
 
@@ -646,7 +657,7 @@ more information on the subject.
 #
 sub build_companion
 {
-	my @coefficients = ($ascending_flag)? reverse @_: @_;
+	my @coefficients = ($ascending_flag == 0)? reverse @_: @_;
 	my $n = $#coefficients - 1;
 	my @h;
 
@@ -655,7 +666,7 @@ sub build_companion
 	#
 	# First step:  Divide by the leading coefficient and negate.
 	#
-	my $cn = - (shift @coefficients);
+	my $cn = - (pop @coefficients);
 	map($_ /= $cn, @coefficients);
 
 	#
@@ -663,7 +674,7 @@ sub build_companion
 	#
 	for my $i (0 .. $n)
 	{
-		$h[$i][$n] = pop @coefficients;
+		$h[$i][$n] = shift @coefficients;
 		map($h[$i][$_] = 0.0, 0 .. $n - 1);
 	}
 
@@ -672,7 +683,6 @@ sub build_companion
 	return @h;
 }
 
-
 =head3 balance_matrix
 
 Balances the matrix (makes the rows and columns have similar norms) created
@@ -1106,7 +1116,7 @@ by returning C<-b/a>. This may be in either a scalar or an array context.
 
 sub linear_roots
 {
-	my($a, $b) = ($ascending_flag)? reverse @_: @_;
+	my($b, $a) = ($ascending_flag == 0)? reverse @_: @_;
 
 	if (abs($a) < $epsilon)
 	{
@@ -1130,7 +1140,7 @@ using the well-known quadratic formula. Returns a two-element list.
 
 sub quadratic_roots
 {
-	my($a, $b, $c) = ($ascending_flag)? reverse @_: @_;
+	my($c, $b, $a) = ($ascending_flag == 0)? reverse @_: @_;
 
 	if (abs($a) < $epsilon)
 	{
@@ -1165,7 +1175,7 @@ complex numbers.
 
 sub cubic_roots
 {
-	my($a, $b, $c, $d) = ($ascending_flag)? reverse @_: @_;
+	my($d, $c, $b, $a) = ($ascending_flag == 0)? reverse @_: @_;
 	my @x;
 
 	if (abs($a) < $epsilon)
@@ -1180,11 +1190,11 @@ sub cubic_roots
 	# temporarily set the flag to zero and reset before returning.
 	#
 	my $temp_ascending_flag = $ascending_flag;
-	$ascending_flag = 0;
+	$ascending_flag = 1;
 
 	if (abs($d) < $epsilon)
 	{
-		@x = quadratic_roots($a, $b, $c);
+		@x = quadratic_roots($c, $b, $a);
 		$ascending_flag = $temp_ascending_flag;
 		return (0, @x);
 	}
@@ -1274,7 +1284,7 @@ type.
 
 sub quartic_roots
 {
-	my($a,$b,$c,$d,$e) = ($ascending_flag)? reverse @_: @_;
+	my($e, $d, $c, $b, $a) = ($ascending_flag == 0)? reverse @_: @_;
 	my @x = ();
 
 	if (abs($a) < $epsilon)
@@ -1286,14 +1296,14 @@ sub quartic_roots
 	#
 	# We're calling exported functions that also check
 	# the $ascending_flag. To avoid reversing the reversed,
-	# temporarily set the flag to zero and reset before returning.
+	# temporarily set the flag to one and reset before returning.
 	#
 	my $temp_ascending_flag = $ascending_flag;
-	$ascending_flag = 0;
+	$ascending_flag = 1;
 
 	if (abs($e) < $epsilon)
 	{
-		@x = cubic_roots($a, $b, $c, $d);
+		@x = cubic_roots($d, $c, $b, $a);
 		$ascending_flag = $temp_ascending_flag;
 		return (0, @x);
 	}
@@ -1338,7 +1348,7 @@ sub quartic_roots
 		#
 		# Special case: h == 0.  We have a cubic times y.
 		#
-		@x = (0, cubic_roots(1, 0, $f, $g));
+		@x = (0, cubic_roots($g, $f, 0, 1));
 	}
 	elsif (abs($g * $g) < $epsilon)
 	{
@@ -1353,7 +1363,7 @@ sub quartic_roots
 		# possible for $g to be outside of epsilon while
 		# $g**2 is inside, i.e., "zero").
 		#
-		my($p, $q) = quadratic_roots(1, $f, $h);
+		my($p, $q) = quadratic_roots($h, $f, 1);
 		$p = sqrt($p);
 		$q = sqrt($q);
 		@x = ($p, -$p, $q, -$q);
@@ -1377,7 +1387,7 @@ sub quartic_roots
 		# quadratics from it.
 		#
 		my $z;
-		($z, undef, undef) = cubic_roots(1, 2*$f, $f*$f - 4*$h, -$g*$g);
+		($z, undef, undef) = cubic_roots(-$g*$g, $f*$f - 4*$h, 2*$f, 1);
 
 		#### $z
 
@@ -1386,8 +1396,8 @@ sub quartic_roots
 		my $beta = ($f + $z - $rho)/2;
 		my $gamma = ($f + $z + $rho)/2;
 
-		@x = quadratic_roots(1, $alpha, $beta);
-		push @x, quadratic_roots(1, -$alpha, $gamma);
+		@x = quadratic_roots($beta, $alpha, 1);
+		push @x, quadratic_roots($gamma, -$alpha, 1);
 	}
 
 	$ascending_flag = $temp_ascending_flag;
@@ -2151,7 +2161,8 @@ sub poly_iteration
 	for my $k (keys %limits)
 	{
 		#
-		# If this is a real iteration limit, save its old value, then set it.
+		# If this is a real iteration limit, save its old
+		# value, then set it.
 		#
 		if (exists $iteration{$k})
 		{
@@ -2223,7 +2234,8 @@ sub poly_tolerance
 	for my $k (keys %tols)
 	{
 		#
-		# If this is a real tolerance limit, save its old value, then set it.
+		# If this is a real tolerance limit, save its old
+		# value, then set it.
 		#
 		if (exists $tolerance{$k})
 		{
@@ -2260,13 +2272,17 @@ sub poly_nonzero_term_count
 }
 
 END {
-	#unless ($ascending_order_called)
-	#{
-	#	warn "Ascending order is in default state.\n",
-	#	"Please use ascending_order() to make sure your function\n",
-	#	"calls will be in the correct order when version 3.00 is installed.\n";
-	#warn "Please see Math::Polynomial::Solve documentation.\n" if ($w == 1);
-	#}
+	unless (1)	#unless ($ascending_order_called)
+	{
+		my $end_list = join("\n", keys %called_by);
+		warn "Coefficient order is in a default state, which will change by version 3.00.\n\n",
+		"Please use ascending_order(0) at the beginning of your code to make\n",
+		"sure your function parameters will be in the correct order when the\n",
+		"default order changes.\n\n",
+		"See the README file and the Math::Polynomial::Solve documentation for\n",
+		"more information.\n",
+		"File(s): $end_list\n";
+	}
 }
 
 1;
@@ -2329,7 +2345,7 @@ polynomials can now be solved, albeit through numeric analysis methods.
 Hiroshi Murakami's Fortran routines were at
 L<http://netlib.bell-labs.com/netlib/>, but do not seem to be available
 from that source anymore. However, his files have been located and are now
-included in the C<references/qralg> directory.
+included in the C<references/qralg> directory of this package.
 
 He referenced the following articles:
 
